#

class Field:

    def __init__(self, **args):
        if 'max_length' in args:
            self.max_length = args['max_length']
        
        if 'value' in args:
            self.value = args['value']
        
        if 'required' in args:
            self.required = args['required']

        if 'types' in args['types']:
            self.type = args['types']

    def get(self):
        if len(self.value) > self.max_length:
            raise ValueError('Max value error ')

        if self.value is None and self.required == True:
            raise ValueError('Value has required')

        if self.type != type(self.value):
            raise TypeError('Valeur non conforme')
        return self.value


class CharField(Field):

    def __init__(self, **args):
        super().__init__(**args)
        
    def clean(cls):
        return cls.get()


import mysql.connector as msql

class DB_MySql :
    
    def __init__(self, **args):
        try:
            self.__db = msql.connect(**args)
        except msql.errors.ProgrammingError as error:
            query = f"""CREATE DATABASE {args['database']} """
            try:
                source_db = msql.connect(host=args['host'], user=args['user'], password=args['password'])
                cursor = source_db.cursor()
                cursor.execute(query)
                cursor.close()
            except Exception as error:
                print(f'Erreur serveur de base de données \n{error}')
            else:
                self.__db = msql.connect(**args)
        else:
            pass
        
    def  execute(self, query, method):
        cursor = self.__db.cursor()
        cursor.execute(query)
        
        if method == 'GET':
            data = []
            for els in cursor:
                data.append(els)
            cursor.close()
            return data
        
        if method == 'POST':
            self.__db.commit()
            cursor.close()


import json

class DataSource:

    @classmethod
    def run(cls):
        quiz = ''
        json_data = {}
        has_finish = True
        all_data = {}

        with open('server/assets/data_source_quiz.json', 'r') as src:
            data = src.read()
            all_data = json.loads(data)

        for item in all_data.keys() :
            quiz = all_data[item]
            values = cls.simple_quiz(quiz)
            json_data[item] = values

        with open(f'server/data_source.json', 'w') as src:
            data_ftd = {
                "host": json_data['host'],
                "user": json_data['user'],
                "password": json_data['password'],
                "dbname": json_data['dbname'],
                "datasource": json_data['datasource'],
            }
            json.dump(data_ftd, src, indent=4)
    
    @classmethod
    def simple_quiz(cls, data):
        resp = input(data)
        return resp


from src.config import SERVER_CONFIG, MODELS_SOURCE_PATH
from .list_db_type import LIST_TYPE
from src.packages import DB_Sqlit, DB_MySql, DB_Mongo

__SERVER__ = None

class Model:
    
    def __init__(self, filedata):
        self.__data__ = filedata
        self.__tablename__ = f"{self.__data__['name']}".lower()
        self.__create__()
   

    def __create__(self):
        global __SERVER__
        if self.__data__ != None :
            self.__data_source = self.__data__['option']['dataSource']
            self.__module_property__ = self.__data__['properties']
            dbname = self.__data__['option']['dbname']
            
            if  self.__data_source == 'db_sqlite_source':
                __SERVER__ = DB_Sqlit(dbname)
                
            if  self.__data_source == 'db_mysql_source':
                __SERVER__ = DB_MySql(**SERVER_CONFIG) 
            
            if  self.__data_source == 'db_mongo_source':
                __SERVER__ = DB_Mongo(**SERVER_CONFIG)  
            
            self.__connect__()     
    
    
    def __connect__(self): 
        try: 
            if  self.__data_source == 'db_mongo_source':
                self.__db_collect =  __SERVER__.create_collection(self.__data__['name'])
            else:
                query = self.__create_query_table__()
                __SERVER__.execute(query, 'POST')
        except Exception as error:
            pass
            # print('Error __connect__', error)


    def execute(self, query, method):
        global __SERVER__
        if  self.__data_source == 'db_mongo_source':
           return __SERVER__.execute(query, method)
        else :
            return __SERVER__.execute(query, method)
    
    
    def __create_query_table__(self):
        property_length = len(self.__module_property__)
        query = f"""CREATE TABLE {self.__tablename__} (\n"""

        if self.__data_source == 'db_mysql_source':
            query += "id int PRIMARY KEY Auto_increment,\n"
        
        if self.__data_source == 'db_sqlite_source':
            query += "id INTEGER PRIMARY KEY AUTOINCREMENT,\n" 
            
        cpt = 0
        for item in self.__module_property__:
            cpt+=1
            propertyItem = self.__module_property__[item]
            default_value = lambda x : 'NOT NULL' if self.get_boolean_value(x) else ''
            index_value = lambda x : 'UNIQUE' if self.get_boolean_value(x) else ''
            list_value = LIST_TYPE[self.__data_source]
            if cpt == property_length:
                query += f"{item} {list_value[propertyItem['type']]}({propertyItem['length']}) {default_value(propertyItem['required'])} {index_value(propertyItem['index'])}"
            else :
                query += f"{item} {list_value[propertyItem['type']]}({propertyItem['length']}) {default_value(propertyItem['required'])} {index_value(propertyItem['index'])},\n"
        
        query+="""\n)"""  
        return query
    

    @classmethod
    def get_classname(cls):
        classname = f'{cls.__class__}'.replace("<class '__main__.", "")
        classname = classname.replace("'>", "").lower()
        return classname


    def get_boolean_value(self, value):
        list_bool = {
            "true" : True,
            "True" : True,
            "False" : False,
            "false" : False
        }

        if value in list_bool.keys():
            return list_bool[value]
        return False


    def get_python_type(self, value):
        list_type = {
            "string" : str,
            "integer" : int,
            "boolean" : bool,
            "float": float
        }
        
        if value in list_type.keys():
            return list_type[value]
        return False


    def gets(self, method='GET',**ctrl):
        """
        Cette methode permet de recuperer toute les occurences
        d une table et elle retourne un dictionnaire

        Exemple :
        voiture = Voiture()
        result = voiture.gets()
        print(result)
        """
        query = f"SELECT * FROM {self.__tablename__}"
        return self.execute(query, method)


    def remove_id(self, _id_):
        """
        Cette methode permet de suprimer une occurence
        d une table et elle retourne la valeur de occurence
        ou False quand elle n'a pas marché

        Exemple :
        voiture = Voiture()
        result = voiture.remove(2)
        print(result)
        """
        result = self.search(key='id', value=_id_)

        if len(result) > 0 :
            query = f"DELETE FROM {self.__tablename__} WHERE id={_id_}"
            print(query)
            self.execute(query, 'POST')
            return result
        else :
            return False

    
    def search_id(self, key, value, **parms):
        """
        Cette methode permet de rechercher une occurence
        specifique  dans la table et elle retourne un dictionnaire

        Exemple :
        voiture = Voiture()
        result = voiture.search(key="marque", value="BMW", limit=(1,10), order="crs")
        print(result)
        """
        if type(value) != str:
            query = f"SELECT * FROM {self.__tablename__} WHERE {key}={value}"
        else :
            query = f'SELECT * FROM {self.__tablename__} WHERE {key}="{value}"'
        return self.execute(query, "GET")
    

    def add(self, method='POST', **args):

        import json

        query = f"INSERT INTO {self.__tablename__} VALUES(NULL, "
        
        try:
            with open(f'{MODELS_SOURCE_PATH}/{self.__tablename__}.json') as src:
                data = src.read()
                jsonData = json.loads(data)
                allsproperties = jsonData['properties']
            
            for key in args.keys():
                if args[key] == None:
                    raise ValueError('Les champs n ont pas bien ete renseigné ')
        
        
            for key in args.keys():
                if key in allsproperties.keys() :
                    attrib = allsproperties[key]
                    typeof = self.get_python_type(attrib['type'])
                    if not (typeof and typeof == type(args[key])):
                        raise ValueError(f"Type non conforme variable {key}\n saisie : {type(args[key])} =>attendu : {attrib['type']}")
                else :
                    raise ValueError(f"Type de variable introuvable")
            
        except Exception as error:
            print("Error ", error)
        else:

            format_values = ""
            cpt = 0
            for key in args.keys():
                cpt+=1
                if cpt>= len(args):
                    format_values += f'"{args[key]}"'
                else :
                    format_values += f'"{args[key]}",'
            
            query+=f"{format_values})"
            print(query)
            self.execute(query, method)
           
    def generate(self, classname):
        global __SERVER__
        list_getted = {
            'execute' : self.execute,
            'remove' : self.remove_id,
            'find' : self.gets,
            'add' : self.add,
            'findOne' : self.search_id,
            '__python_type__' : self.get_python_type,
            '__tablename__' : classname,
            'db_connect' : __SERVER__,
            }
        return list_getted


import sys
import os
from .moduls import Modul
from .datasource import DataSource

class Server:

    @classmethod
    def run(cls):
        args = sys.argv
        if len(args) > 1 :
            params = args[1]
            if params == 'model':
                Modul.run()
            if params == 'run':
                os.system('python main.py')
            if params == 'datasource':
                DataSource.run()



LIST_TYPE = {
    'db_mysql_source' : {
        'string' : 'Varchar',
        'integer' : 'Int',
        'text' : 'Text',
    },
    'db_sqlite_source' : {
        'string' : 'Text',
        'integer' : 'Integer'
    }
}


# Server connect property
PARAM = {}
import json

with open('src/server/data_source.json') as src:
    data = src.read()
    PARAM = json.loads(data)


SERVER_CONFIG = {
    'database': PARAM['dbname'], 
    'host': PARAM['host'], 
    'user': PARAM['user'], 
    'password': PARAM['password']
}

LOCAL_SQLITE_DB = "bin/"

MODELS_SOURCE_PATH = "clients"


import json
from . import Model
from src.config import MODELS_SOURCE_PATH

class MetaModel(type):

    def __new__(metacls, classname, herited, contants):
        clsname = classname.lower()
        propertys = {}
        
        if 'model' in clsname :
            return type.__new__(metacls, clsname, herited, contants)
        else :
            with open(f'{MODELS_SOURCE_PATH}/{clsname}.json', 'r') as src:
                data = src.read()
                propertys = json.loads(data)
                model = Model(propertys)
                methods = model.generate(clsname)
                for meth in methods.keys():
                    propertys['properties'][meth]=methods[meth]

            # try:
            
            # except Exception as error:
            #     print(f'<:: debug ::> file {clsname} not exist <!>')
            # else :
                if clsname != f"{propertys['name']}".lower():
                    raise ValueError('Nom de la classe est non comforme a celle de la table')
                return type.__new__(metacls, clsname, herited, propertys['properties'])

    def __init__(cls, classname, herited, contants):
        type.__init__(cls, classname, herited, contants)

    def toggler(cls):
        pass


from src.packages.db_sqlite import DB_Sqlit
from src.packages.db_mysql import DB_MySql
from src.packages.db_mongo import DB_Mongo
from src.packages.db_models import Model
from src.packages.db_controller import MetaModel


import sqlite3
from src.config import LOCAL_SQLITE_DB as local_db_file

class DB_Sqlit :
    
    def __init__(self, dbname):
        self.__db = sqlite3.connect(f'{local_db_file}/{dbname}.db')

    def execute(self, query, method):
        cursor = self.__db.cursor()
        
        if method == 'GET':  
            resp = cursor.execute(query)
            return resp.fetchall()
    
        if method == 'POST':
            cursor.execute(query)
            cursor.close()
            self.__db.commit()


from pymongo import MongoClient

class DB_Mongo:
    
    def __init__(self, **args):
        db = MongoClient(host=args['host'], port=27017)
        self.__db = db[args['dbname']]
            
    
    def create_collection(self, collection):
        self.__db[collection]
       
    def  execute(self, collection, values, method):
        db_collect = self.__db[collection]
        if method == 'GET':
            data = db_collect.find()
            return data
        
        if method == 'POST':
            db_collect.insert_one(values)


import json
from src.config import MODELS_SOURCE_PATH

class Modul:

    @classmethod
    def run(cls):
        quiz = ''
        json_data = {}
        has_finish = True
        all_data = {}

        with open('src/server/assets/moduls_quiz.json', 'r') as src:
            data = src.read()
            all_data = json.loads(data)

        item_data = {}

        for item in all_data.keys() :
            quiz = all_data[item]
            if type(quiz) == dict:
                if item == 'properties':
                    property_name = 'null'
                    while property_name != '':
                        property_name = input('Nom de la propriete : ')
                        if property_name != '' :
                            values = cls.dict_quiz(quiz)
                            item_data[property_name] = values
                            # print(item_data)
                            # json_data[item][property_name] = values
                else :
                    values = cls.dict_quiz(quiz)
                    json_data[item] = values
            else :
                values = cls.simple_quiz(quiz)
                json_data[item] = values
        
        json_data['properties'] = item_data
        file_name = f"{json_data['name']}".lower()
        # json_formated = json.load(json_data)
        

        #Operation de enregistrement propriétées du module en format json
        with open(f'{MODELS_SOURCE_PATH}/{file_name}.json', 'a') as src:
            data_ftd = str(json_data)
            data_ftd = data_ftd.replace("'","\"")
            src.write(data_ftd)
            # json.dump(data_ftd, src, indent=4)
        

        #Operation d'inclusion du module dans le programme 
        with open(f'src/modules/lunch.py', 'a+') as src:
            class_property = ''
            for item in json_data['properties']:
                class_property+=f'\t\tself.{item} = None\n'

            discrip = '"""\n\t-> get()\n\t-> add()\n\t-> findOne()\n\t-> find()\n\t-> remove()\n\t-> execute()\n\t"""'
            script_of_moduel = f"\n\nclass {file_name.capitalize()}(metaclass=MetaModel):\n"
            script_of_moduel+=f"\t{discrip}\n"
            script_of_moduel+="\tdef __init__(self):\n"
            script_of_moduel+=f"{class_property}"
            src.write(script_of_moduel)
        

        with open(f'{MODELS_SOURCE_PATH}/{file_name}.py', 'a') as src:
            
            doc_model = '\t"""\n'
            for item in json_data['properties']:
                doc_model+=f"\t\# {item} <{json_data['properties'][item]['type']}>\n"
            doc_model += '\t"""\n'

            model_meth = "\tdef save(self):\n"
            model_meth+= "\t\tpass\n"
            model_meth+= "\tdef put(self):\n"
            model_meth+= "\t\tpass\n"
            model_meth+= "\tdef get(self):\n"
            model_meth+= "\t\tpass\n"
            model_meth+= "\tdef delete(self):\n"
            model_meth+= "\t\tpass\n"

            script_of_moduel = f"from src.modules import {file_name.capitalize()}"
            script_of_moduel+= f"\n\nclass {file_name.capitalize()}Model({file_name.capitalize()}):\n"
            script_of_moduel+=f"{doc_model}"
            script_of_moduel+=f"{model_meth}"
            src.write(script_of_moduel)



    @classmethod
    def get_python_type(cls, value):
        list_type = {
            "string" : str,
            "integer" : int,
            "boolean" : bool,
            "float": float
        }
    
    @classmethod
    def dict_quiz(cls, data):
        resp = {}
        val = None
        for els in data.keys() :
            question = data[els]
            val = input(question)
            if val == "True" or val == "False":
                resp[els] = val.lower()
            resp[els] = val       
        return resp
                

    @classmethod
    def simple_quiz(cls, data):
        resp = input(data)
        return resp